﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Net;
using System.Web;
using System.IO;
using System.Collections;

namespace WebIM_StressTesting
{
    class Program
    {
        static void Main(string[] args)
        {
            Console.Write("Thread Count:");
            String temp = Console.ReadLine();
            Int32 count = Int32.Parse(temp);

            List<WebIM_Client> clients=new List<WebIM_Client>();
            for (int i = 0; i < count; i++)
            {
                WebIM_Client client = new WebIM_Client();
                clients.Add(client);
                client.Start();
            }

            bool completed = false;
            do
            {
                completed = true;
                foreach (WebIM_Client client in clients)
                {
                    if (client.IsCompleted == false) completed = false;
                }
                Thread.Sleep(2000);
            } while (!completed);

            int sendDelay = 0, sendErrorCount = 0, sendTotal = 0;
            int recvDelay = 0, recvErrorCount = 0, recvTotal = 0;

            foreach (WebIM_Client client in clients)
            {
                sendDelay += client.SendDelayAverage;
                sendErrorCount += client.SendErrorCount;
                sendTotal += client.SendTotal;

                recvDelay += client.RecvDelayAverage;
                recvErrorCount += client.RecvErrorCount;
                recvTotal += client.RecvTotal;
            }

            Console.WriteLine(String.Format("发送总消息数:{0}", sendTotal));
            Console.WriteLine(String.Format("发送平均延迟:{0} ms", sendDelay / clients.Count));
            Console.WriteLine(String.Format("发送错误连接总数:{0}", sendErrorCount));

			Console.WriteLine(String.Format("接收总消息数:{0}", recvTotal));
            Console.WriteLine(String.Format("接收平均延迟:{0} ms", recvDelay / clients.Count));
            Console.WriteLine(String.Format("接收错误连接总数:{0}", recvErrorCount));

            Console.Read();
        }
    }

    class WebIM_Client
    {
        Thread m_ReceiveThread, m_SendThread;
        bool m_ReceiveThreadCompleted = true, m_SendThreadCompleted = false;
        String m_Name = Guid.NewGuid().ToString().ToUpper();

        List<Double> m_SendDelays = new List<Double>();
        Int32 m_SendErrorCount = 0, m_SendTotal = 0;

        List<Double> m_RecvDelays = new List<Double>();
        Int32 m_RecvErrorCount = 0, m_RecvTotal = 0;

        public Int32 SendTotal
        {
            get { return m_SendTotal; }
        }

        public Int32 SendErrorCount
        {
            get { return m_SendErrorCount; }
        }

        public Int32 SendDelayAverage
        {
            get
            {
                Double total = 0;
                foreach (Double d in m_SendDelays) total += d;
                return (Int32)(total / m_SendDelays.Count);
            }
        }

        public Int32 RecvTotal
        {
            get { return m_RecvTotal; }
        }

        public Int32 RecvErrorCount
        {
            get { return m_RecvErrorCount; }
        }

        public Int32 RecvDelayAverage
        {
            get
            {
                Double total = 0;
                foreach (Double d in m_RecvDelays) total += d;
                return (Int32)(total / m_RecvDelays.Count);
            }
        }

        void ReceiveThreadEntry()
        {
            DateTime from = DateTime.Now;
            while (true)
            {
                Hashtable data = new Hashtable();
                data["Sender"] = "*";
                data["Receiver"] = "test_group";
                data["From"] = from;

                try
                {
                    Hashtable ret = Utility.ParseJson(Post("http://localhost/WebIM/recevie.aspx", Utility.RenderJson(data))) as Hashtable;

                    DateTime now = new DateTime((DateTime.Now.Ticks / 10000) * 10000);

                    foreach(object obj in ret["Messages"] as object[])
                    {
                        Hashtable msg = obj as Hashtable;
                        DateTime ct = (DateTime)msg["CreatedTime"];
                        m_RecvDelays.Add((now - ct).TotalMilliseconds);
                        m_RecvTotal++;
                        if (ct > from) from = ct;
                    }
                }
                catch
                {
                    m_RecvErrorCount++;
                }

                Thread.Sleep(100);
            }
        }

        void SendThreadEntry()
        {
            int count = 0;
            while (count < 50)
            {
                Hashtable data = new Hashtable();
                data["Sender"] = m_Name;
                data["Receiver"] = "test_group";
                data["Content"] = "测试消息";
                DateTime now = new DateTime((DateTime.Now.Ticks / 10000) * 10000);

                try
                {
                    Hashtable ret = Utility.ParseJson(Post("http://localhost/WebIM/send.aspx", Utility.RenderJson(data))) as Hashtable;
                    m_SendDelays.Add(((DateTime)ret["CreatedTime"] - now).TotalMilliseconds);
					m_SendTotal++;
                }
                catch
                {
                    m_SendErrorCount++;
                }

                count++;
                Thread.Sleep(1000);
            }
            m_SendThreadCompleted = true;
        }

        private static string Post(String url,String content)
        {
            HttpWebRequest req = HttpWebRequest.Create(url) as HttpWebRequest;
            req.Timeout = 30 * 1000;
            req.Method = "POST";
            req.ContentType = "application/x-www-form-urlencoded";

            Byte[] buffer = Encoding.UTF8.GetBytes(content);
            req.ContentLength = buffer.Length;
            req.GetRequestStream().Write(buffer, 0, buffer.Length);
            req.GetRequestStream().Close();

            HttpWebResponse response = req.GetResponse() as HttpWebResponse;
            Stream stream = response.GetResponseStream();

            MemoryStream ms = new MemoryStream();
            Byte[] temp = new Byte[4096];
            int count = 0;
            while ((count = stream.Read(temp, 0, 4096)) > 0)
            {
                ms.Write(temp, 0, count);
            }
            stream.Close();

            string str = Encoding.GetEncoding(response.CharacterSet).GetString(ms.GetBuffer());
            return str;
        }

        public void Start()
        {
            m_ReceiveThread = new Thread(ReceiveThreadEntry);
            m_SendThread = new Thread(SendThreadEntry);

            m_ReceiveThread.Start();
            m_SendThread.Start();
        }

        public bool IsCompleted
        {
            get { return m_SendThreadCompleted && m_ReceiveThreadCompleted; }
        }
    }
}
